<?php
declare(strict_types=1);


namespace Lewisdale\App\Models\Data;

use Doctrine\Common\Collections\Collection;
use Doctrine\DBAL\Types\Types;
use Doctrine\ORM\Mapping as ORM;
use DOMXPath;
use Lewisdale\App\Models\Types\XMLElement;
use Lewisdale\App\Requests\Robots;
use Ramsey\Uuid\Doctrine\UuidGenerator;
use Ramsey\Uuid\UuidInterface;
use SimpleXMLElement;
use function Symfony\Component\String\s;

#[ORM\Entity]
#[ORM\Table(name: 'feeds')]
#[ORM\HasLifecycleCallbacks]
class Feed
{
    #[ORM\Id]
    #[ORM\Column(type: "uuid", unique: true)]
    #[ORM\GeneratedValue(strategy: "CUSTOM")]
    #[ORM\CustomIdGenerator(class: UuidGenerator::class)]
    public UuidInterface $id;

    #[ORM\Column(type: 'string')]
    public string $url;

    #[ORM\Column(type: 'string')]
    public string $title;


    /**
     * @var Collection<FeedFilter>
     */
    #[ORM\OneToMany(mappedBy: 'feed', targetEntity: FeedFilter::class, cascade: ['persist', 'remove'])]
    public Collection $feedFilters;

    #[ORM\Column(type: XMLElement::NAME, nullable: true)]
    public SimpleXMLElement|null $remoteFeed;

    #[ORM\Column(type: XMLElement::NAME, nullable: true)]
    public SimpleXMLElement|null $filteredFeed;

    public function fetch(): void
    {
        if (Robots::allowed($this->url)) {
            $this->remoteFeed = simplexml_load_file($this->url);
        } else {
            throw new \Exception("Robots.txt disallows fetching this feed");
        }
    }

    private function test_item(\DOMNode $item): bool {
        foreach ($this->feedFilters as $filter) {
            if (!$filter->execute(simplexml_import_dom($item))) {
                return false;
            }
        }
        return true;
    }

    public function filter() : void
    {
        if ($this->remoteFeed === null) {
            $this->fetch();
        }

        $dom = dom_import_simplexml($this->remoteFeed);
        $doc = $dom->ownerDocument;

        $xpath = new DOMXPath($doc);
        $filter_queries = implode("|", array_map(fn($filter) => $filter->to_xpath(), $this->feedFilters->toArray()));

        $xpath_query = '//item[' . $filter_queries . ']';

        $to_remove = $xpath->query($xpath_query);
        echo "Removing " . $to_remove->length . " items\n";
        foreach ($to_remove as $item) {
            $item->parentNode->removeChild($item);
        }

        $this->filteredFeed = simplexml_import_dom($dom);
    }
}