<?php
declare(strict_types=1);


namespace Lewisdale\App\Tools\Console;

use Symfony\Component\Console\{Attribute\AsCommand,
    Command\Command,
    Input\InputArgument,
    Input\InputInterface,
    Output\OutputInterface};
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\EntityManager;
use Lewisdale\App\Models\Data\Feed;
use Lewisdale\App\Models\Data\FeedFilter;
use Lewisdale\App\Models\Data\FilterTarget;
use Lewisdale\App\Models\Data\FilterType;
use Lewisdale\App\Models\Repositories\FeedRepository;
use function DI\add;

#[AsCommand(name: "test:feed")]
class TestFeed extends Command
{
    public function __construct(private readonly EntityManager $em,
                                private readonly FeedRepository $feedRepository)
    {
        parent::__construct();
    }

    protected function configure(): void
    {
        $this
            ->addArgument("url", InputArgument::REQUIRED, "The URL of the feed to test")
            ->addArgument("title", InputArgument::REQUIRED, "The title of the feed to test");
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $output->writeln([
            "Testing Feed",
            "============",
        ]);

        $url = $input->getArgument("url");
        $title = $input->getArgument("title");

//        $output->writeln("Setting up a new feed for: $url");
//
//        $feed = new Feed();
//        $feed->url = $url;
//        $feed->title = $title;
//        $feed->fetch();
//        $feed->feedFilters = new ArrayCollection();
//        $feed->feedFilters->add( new FeedFilter(FilterTarget::TITLE, FilterType::INCLUDE, "[No Ads]", $feed));
//
//        $this->em->persist($feed);
//        $this->em->flush();
//
//        $cnt = $this->feedRepository->count([]);
//        $output->writeln("Feed count: $cnt");

        $saved_feed = $this->feedRepository->findOneBy(['url' => $url, 'title' => $title]);
        $output->writeln("Feed title: " . $saved_feed->title);

        $s2_filter = new FeedFilter(FilterTarget::TITLE, FilterType::EXCLUDE, "S2", $saved_feed);
        $this->em->persist($s2_filter);
        $this->em->flush();

        $saved_feed = $this->feedRepository->findOneBy(['url' => $url, 'title' => $title]);
        $saved_feed->filter();

        $this->em->persist($saved_feed);
        $this->em->flush();

        $output->writeln("Filtered feed: " . $saved_feed->filteredFeed->asXML('feed_filtered.xml'));

        return Command::SUCCESS;
    }
}